//---------------------------------------------------------------------------
#include "bitmap.h"

//---------------------------------------------------------------------------

/* Routines for counting the number of bits in a 32 byte word */

/*
 * This routine counts bits by dividing the word into 1-bit fields and adding
 * Pairs of adjacent fields together.  This gives 2-bit fields, each containing a
 * count of the number of bits that were originally in those positions.
 * Now you can add adjacent fields together to get 4 bit fields.  Then 8, 16, and
 * finally 32.
 */
unsigned Bitmap::count() const
{
	register unsigned tmp;
    register unsigned value = val;

	tmp = value >> 1;
	value &= 0x55555555;
	tmp &= 0x55555555;
	value += tmp;

	/*
	 * The word is now broken into fields of two bits each.
	 * Each field contains the number of bits that were set
	 */

	tmp = value >> 2;
	value &= 0x33333333;
	tmp &= 0x33333333;
	value += tmp;				/* now each field is 4 bits wide */

	tmp = value >> 4;
	value &= 0x0f0f0f0f;
	tmp &= 0x0f0f0f0f;
	value += tmp;				/* now 8 */

	tmp = value >> 8;
	value &= 0x00ff00ff;
	tmp &= 0x00ff00ff;
	value += tmp;				/* now 16 */

	tmp = value >> 16;
	value &= 0x0000ffff;
	tmp &= 0x0000ffff;
	return value + tmp;			/* finally one field 32 bits wide */
}

#if 0
AnsiString
Bitmap::to() const
{
    AnsiString result("[");
    bool first=true;

    for (int i=0; i<10; ++i) {
        if (check(i)) {
            if (!first) {
                result += ',';
            }
            first = false;
            result += i;
        }
    }
    result += ']';
    return result;
}
#endif
