//
// This file contains the glue between C code and C++ code.  The file
// is compiled with the C++ compiler, but the main interface is the
// solve_sudoku() funtion, which is compiled with C linkage.


// Include the hpgcc49.h file with C linkage.  We need this file for
// malloc() and free()
extern "C" {
#include <hpgcc49.h>
}

#include "Board.h"

Board board;



// Here is the main routine.  It takes an 81 character string
// and populates a sudoku board with it.  Any non-digits in the string
// correspond to blanks on the board.  The function attempts to solve
// the puzzle.  It replaces str with the solved puzzle and returns 1
// on success and 2 if the puzzle can't be solved.
extern "C"
int solve_sudoku(char *str)
{
    board.fromString(str);	// copy string into board
    int result = board.solve() ? 1 : 2; // attempt to solve
    board.toString(str);	// convert to string
    return result;
}


////////////////////////////////////////////////////////////////////////
// below are the C++ operators that we need to define ourselves.
///////////////////////////////////////////////////////////////////////

void * operator new(unsigned long size)
{
    void *ptr=malloc(size);
    return ptr;
}

void * operator new[](unsigned long size)
{
    void *ptr=malloc(size);
    return ptr;
}

void operator delete(void *p)
{
    free(p);
}
